#!/usr/bin/bash

# $Author: jeffr $
# $Date: 2022-11-11 10:15:58 -0500 (Fri, 11 Nov 2022) $
# $Revision: 25 $

# Make sure radar ID and animation period are provided. Delete is optional
if [ $# -lt 2 ] ; then
	echo "Usage $0 {radar ID} {animation duration in mins} {delete files in archive}"
	exit 1
fi

# List of radar ids (https://www.weather.gov/jetstream/ridge_download)
radar_ids=( ABC ABR ABX ACG AEC AHG AIH AKC AKQ AMA AMX APD APX ARX ATX BBX BGM BHX BIS BLX BMX BOX BRO BUF BYX CAE CBW CBX CCX CLE CLX CRP CXX CYS DAX DDC DFX DGX DIX DLH DMX DOX DTX DVN DYX EAX EMX ENX EOX EPZ ESX EVX EWX EYX FCX FDR FDX FFC FSD FSX FTG FWS GGW GJX GLD GRB GRK GRR GSP GUA GWX GYX HDX HGX HKI HKM HMO HNX HPX HTX HWA ICT ICX ILN ILX IND INX IWA IWX JAX JGX JKL JUA LBB LCH LGX LIX LNX LOT LRX LSX LTX LVX LWX LZK MAF MAX MBX MHX MKX MLB MOB MPX MQT MRX MSX MTX MUX MVX MXX NKX NQA OAX OHX OKX OTX PAH PBZ PDT POE PUX RAX RGX RIW RLX RTX SFX SGF SHV SJT SOX SRX TBW TFX TLH TLX TWX TYX UDX UEX VAX VBX VNX VTX VWX YUX )

# Was a valid radar ID provided?
radar_flag=0
for i in "${radar_ids[@]}"
do
	if [ $1 = $i  ] ; then
		radar_flag=1
	fi
done

# Radar ID not found
if [ $radar_flag -eq 0 ] ; then
	echo "Radar ID \"$1\" not found. Valid radar IDs include:"
    echo ${radar_ids[@]}
	echo Refer to https://www.weather.gov/jetstream/ridge_download
	echo Quitting.
	exit 1
fi

#
# Basic variables for processing
#

footer_len=1               # Number of lines in footer to cut off
image_freq=4               # How often radar images are updated by NWS
resize_amt='25%'           # Amount to shrink the images by
animate_dur=$2             # Duration (in minutes) to depict in animation
radar_bg='background.gif'  # Background image (e.g. map) to use
font_color='white'         # Font color to use for radar image date and time

# Number of files to download
num_files=$(($animate_dur / $image_freq))
# Radar images reside at this URL
img_url="https://mrms.ncep.noaa.gov/data/RIDGEII/L2/K$1/BREF_RAW/"

#
# Get the list of radar image files and download files
#

# Temporary files for the data
temp=$(mktemp /tmp/radar.XXXXXXXXX)
temp2=$(mktemp /tmp/radar.XXXXXXXXX)
temp3=$(mktemp /tmp/radar.XXXXXXXXX)

# Get the entire list of files
lynx -dump -nolist $img_url > $temp

# Process the list so it only contains the needed files
tail -n $(($num_files + $footer_len)) $temp > $temp2
head -n $num_files $temp2 | awk -F" " '{ print $1}' > $temp3

# Download the files if necessary
for file in $( cat $temp3 )
do
	# Has the file already been downloaded?
	if [ ! -f archive/$file ]; then
		# Download file
		wget -q $img_url$file -P archive
	fi
done

#
# Process image files
#

for file in $( cat $temp3 )
do
	# Copy to temp dir
	cp archive/$file tmp/$file
	# Decompress file
	gzip -df tmp/$file

	# Process file
	base_name=$(basename $file .tif.gz)
	# Resize and convert
	convert -quiet tmp/$base_name.tif -resize $resize_amt tmp/$base_name.gif
	# Combine with background
	convert -quiet $radar_bg tmp/$base_name.gif -composite +dither -colors 128 tmp/$base_name.gif
	# Add date and time label
	file_date=$(date -r archive/$file)
	mogrify -quiet -gravity SouthEast -pointsize 20 -fill $font_color -annotate +0+100 "$file_date ($1)" tmp/$base_name.gif

	# Delete original
	rm tmp/$base_name.tif
done

# Copy last(latest) file and create animation
cp tmp/$base_name.gif radar.gif
convert -quiet -delay 20 -loop 0 tmp/*.gif +dither -fuzz 25% -layers optimize-plus animation.gif

#
# Clean up
#

# Delete files in tmp dir
rm tmp/*.gif

# Delete radar images in archive
if [ ! -z $3 ] && [ $3 = "delete" ] ; then
	rm ./archive/*
fi

rm $temp $temp2 $temp3
exit 0
